@@ -31,21 +31,24 @@
 #include "theme.h"
 #include "view.h"
 #include "document.h"
+#include "Hposdlg.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
 #include "libgcpcanvas/gcp-canvas-pango.h"
-#include <math.h>
 #include <gcu/element.h>
 #include <openbabel/mol.h>
 #include <glib/gi18n-lib.h>
+#include <cmath>
 
 using namespace gcu;
 
 #define ATOM_EPSILON 0.1
 
 gcpAtom::gcpAtom(): Atom(),
-	m_ShowSymbol (false)
+	DialogOwner (),
+	m_ShowSymbol (false),
+	m_HPosStyle (AUTO_HPOS)
 {
 	m_Valence = -1; //unspecified
 	m_nlp = 0;
@@ -73,10 +76,11 @@ gcpAtom::~gcpAtom()
 	gcpView *pView = pDoc->GetView ();
 	map<string, Object*>::iterator i;
 	Object* electron = GetFirstChild (i);
-	while (electron){
+	while (electron) {
 		pView->Remove (electron);
+		electron->SetParent (NULL); // avoids a call to RemoveElectron()
 		delete electron;
-		electron = GetNextChild (i);
+		electron = GetFirstChild (i);
 	}
 	if (m_Layout)
 		g_object_unref (G_OBJECT (m_Layout));
@@ -85,7 +89,9 @@ gcpAtom::~gcpAtom()
 }
 
 gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z),
-	m_ShowSymbol (false)
+	DialogOwner (),
+	m_ShowSymbol (false),
+	m_HPosStyle (AUTO_HPOS)
 {
 	m_ChargeAuto = false;
 	m_HPos = GetBestSide();
@@ -105,7 +111,9 @@ gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z),
 }
 
 gcpAtom::gcpAtom(OBAtom* atom): Atom(),
-	m_ShowSymbol (false)
+	DialogOwner (),
+	m_ShowSymbol (false),
+	m_HPosStyle (AUTO_HPOS)
 {
 	m_x = atom->GetX();
 	m_y = - atom->GetY();
@@ -135,7 +143,7 @@ void gcpAtom::SetZ(int Z)
 	Atom::SetZ(Z);
 	m_Element = Element::GetElement (m_Z);
 	if ((m_Valence = m_Element->GetDefaultValence ()))
-		m_HPos = GetBestSide();
+		m_HPos = (m_HPosStyle == AUTO_HPOS)? GetBestSide(): m_HPosStyle;
 	else
 		m_nH = 0;
 	int max = m_Element->GetMaxValenceElectrons ();
@@ -206,7 +214,7 @@ void gcpAtom::Update ()
 		m_Charge = 0;
 		m_ChargeAuto = false;
 	}
-	if (m_ChargeAutoPos) {
+	if (m_ChargeAutoPos && m_ChargePos != 0xff) {
 		NotifyPositionOccupation (m_ChargePos, false);
 		m_ChargePos = 0xff;
 	}
@@ -250,7 +258,7 @@ void gcpAtom::Update ()
 				m_ChargeAuto = true;
 			}
 		}
-		m_HPos = GetBestSide();
+		m_HPos = (m_HPosStyle == AUTO_HPOS)? GetBestSide(): m_HPosStyle;
 	} else {
 		m_nH = 0;
 		if (m_ChargeAuto || !m_Charge) {
@@ -1156,8 +1164,8 @@ void gcpAtom::SetSelected(GtkWidget* w, int state)
 	
 	switch (state) {	
 	case SelStateUnselected:
-		color = "white";
-		chargecolor = "black";
+		color = (char*) "white";
+		chargecolor = (char*) "black";
 		break;
 	case SelStateSelected:
 		chargecolor = color = SelectColor;
@@ -1169,8 +1177,8 @@ void gcpAtom::SetSelected(GtkWidget* w, int state)
 		chargecolor = color = DeleteColor;
 		break;
 	default:
-		color = "white";
-		chargecolor = "black";
+		color = (char*) "white";
+		chargecolor = (char*) "black";
 		break;
 	}
 	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
@@ -1445,6 +1453,18 @@ void gcpAtom::AddElectron (gcpElectron* electron)
 	Update ();
 }
 
+void gcpAtom::RemoveElectron (gcpElectron* electron)
+{
+	// remove the electron from children so that it is not taken into account when
+	// updating.
+	electron->SetParent (NULL);
+	Update ();
+	// Force view update.
+	gcpDocument *pDoc = reinterpret_cast<gcpDocument*> (GetDocument ());
+	if (pDoc)
+		pDoc->GetView ()->Update (this);
+}
+
 void gcpAtom::NotifyPositionOccupation (unsigned char pos, bool occupied)
 {
 	if (occupied)
@@ -1472,31 +1492,31 @@ xmlNodePtr gcpAtom::Save (xmlDocPtr xml)
 		if (m_ChargePos) {
 			switch (m_ChargePos) {
 			case CHARGE_NE:
-				buf = "ne";
+				buf = (char*) "ne";
 				break;
 			case CHARGE_NW:
-				buf = "nw";
+				buf = (char*) "nw";
 				break;
 			case CHARGE_N:
-				buf = "n";
+				buf = (char*) "n";
 				break;
 			case CHARGE_SE:
-				buf = "se";
+				buf = (char*) "se";
 				break;
 			case CHARGE_SW:
-				buf = "sw";
+				buf = (char*) "sw";
 				break;
 			case CHARGE_S:
-				buf = "s";
+				buf = (char*) "s";
 				break;
 			case CHARGE_E:
-				buf = "e";
+				buf = (char*) "e";
 				break;
 			case CHARGE_W:
-				buf = "w";
+				buf = (char*) "w";
 				break;
 			default:
-				buf = "def"; // should not occur
+				buf = (char*) "def"; // should not occur
 			}
 			xmlNewProp (node, (xmlChar*) "charge-position", (xmlChar*) buf);
 		} else {
@@ -1513,6 +1533,10 @@ xmlNodePtr gcpAtom::Save (xmlDocPtr xml)
 	if (GetZ () == 6 && m_ShowSymbol) {
 		xmlNewProp (node, (xmlChar*) "show-symbol", (xmlChar*) "true");
 	}
+	if (m_HPosStyle != AUTO_HPOS) {
+		xmlNewProp (node, (xmlChar*) "H-positon",
+					(xmlChar*) ((m_HPosStyle == LEFT_HPOS)? "left": "right"));
+	}
 	return node;
 }
 	
@@ -1586,6 +1610,15 @@ bool gcpAtom::Load (xmlNodePtr node)
 			m_ShowSymbol = true;
 		xmlFree (buf);
 	}
+	// Load H atoms position if any
+	buf = (char*) xmlGetProp (node, (xmlChar*) "H-position");
+	if (buf) {
+		if (!strcmp (buf, "left"))
+			m_HPosStyle = LEFT_HPOS;
+		else if (!strcmp (buf, "right"))
+			m_HPosStyle = RIGHT_HPOS;
+		xmlFree (buf);
+	}
 	return true;
 }
 	
@@ -1680,21 +1713,45 @@ static void do_display_symbol (GtkToggleAction *action, gcpAtom* Atom)
 	Doc->GetView ()->Update (Atom);
 }
 
-bool gcpAtom::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
+static void do_choose_H_pos (gcpAtom* Atom)
+{
+	new gcpHPosDlg (static_cast<gcpDocument*> (Atom->GetDocument ()), Atom);
+}
+
+bool gcpAtom::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y)
 {
 	bool result = false;
+	GtkActionGroup *group = NULL;
+	GtkAction *action;
 	if (GetZ () == 6 && GetBondsNumber() != 0) {
-		GtkActionGroup *group = gtk_action_group_new ("atom");
-		GtkAction *action;
+		group = gtk_action_group_new ("atom");
 		action = gtk_action_new ("Atom", _("Atom"),NULL, NULL);
 		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
 		action = GTK_ACTION (gtk_toggle_action_new ("show-symbol", _("Display symbol"),  _("Whether to display carbon atom symbol or not"), NULL));
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), m_ShowSymbol);
 		g_signal_connect (action, "toggled", G_CALLBACK (do_display_symbol), this);
 		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Atom'><menuitem action='show-symbol'/></menu></popup></ui>", -1, NULL);
-		gtk_ui_manager_insert_action_group (UIManager, group, 0);
 		result = true;
 	}
-	return result | GetParent ()->BuildContextualMenu (UIManager, object);
+	if (m_nH) {
+		if (!group) {
+			group = gtk_action_group_new ("atom");
+			action = gtk_action_new ("Atom", _("Atom"),NULL, NULL);
+			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
+		}
+		action = GTK_ACTION (gtk_action_new ("H-position", _("Hydrogen atoms position"),  NULL, NULL));
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_choose_H_pos), this);
+		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
+		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Atom'><menuitem action='H-position'/></menu></popup></ui>", -1, NULL);
+	}
+	if (group) {
+		gtk_ui_manager_insert_action_group (UIManager, group, 0);
+		g_object_unref (group);
+	}
+	return result | GetParent ()->BuildContextualMenu (UIManager, object, x, y);
 }
