@@ -32,12 +32,13 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
+#include <cstdlib>
 #include <map>
 #include <string>
 
 using namespace std;
 
-typedef struct _GcpFontSel {
+struct _GcpFontSel {
 	GtkBin base;
 	GtkEntry *m_SizeEntry;
 	GtkListStore *FamilyList, *FaceList, *SizeList;
@@ -301,7 +302,7 @@ gcp_font_sel_class_init (GcpFontSelClass *klass)
 			(GParamFlags) G_PARAM_READWRITE));
 	g_object_class_install_property (object_class, FONT_SEL_PROP_STRETCH,
 		g_param_spec_int ("stretch", _("Stretch"),
-			_("The font stretch (normal, oblique or italic)"),
+			_("The font stretch (condensed, normal or expanded)"),
 			PANGO_STRETCH_ULTRA_CONDENSED, PANGO_STRETCH_ULTRA_EXPANDED, PANGO_STRETCH_NORMAL,
 			(GParamFlags) G_PARAM_READWRITE));
 	g_object_class_install_property (object_class, FONT_SEL_PROP_SIZE,
