@@ -393,11 +393,15 @@ void gcpView::OnDestroy(GtkWidget* widget)
 	else delete m_pDoc;
 }
 
-GnomeCanvasItem* gcpView::GetCanvasItem(GtkWidget* widget, Object* Object)
+GnomeCanvasItem* gcpView::GetCanvasItem (GtkWidget* widget, Object* Object)
 {
-	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(widget), "data");
-	if ((!pData) || (pData->View != this)) return NULL;
-	return (GnomeCanvasItem*) pData->Items[Object];
+	gcpWidgetData* pData = reinterpret_cast<gcpWidgetData*> (g_object_get_data (G_OBJECT (widget), "data"));
+	if ((!pData) || (pData->View != this))
+		return NULL;
+	GnomeCanvasItem *result = reinterpret_cast<GnomeCanvasItem*> (pData->Items[Object]);
+	if (!result)
+		pData->Items.erase (Object);
+	return result;
 }
 
 void gcpView::Print(GnomePrintContext *pc, gdouble width, gdouble height)
